@@ -46,6 +46,7 @@ import sys
 import re
 import os
 import inspect
+import itertools
 import traceback
 import codecs
 from errormator_client.client import PY3
@@ -72,6 +73,43 @@ class _Missing(object):
 
 _missing = _Missing()
 
+def truncate_str(input):
+    return input if len(input) < 255 else input[:255] + '...'
+
+def serialize_to_unicode(input, treat_as_class=False):
+    if hasattr(input, 'iterkeys'):
+        dict_method = input.iterkeys
+    elif hasattr(input, 'keys'):
+        dict_method = input.keys
+    else:
+        dict_method = None
+    if dict_method is not None:
+        # avoid sending environ
+        if input.get('wsgi.version'):
+            return dict([['environ', '<environ-skipped>']])
+        dict_keys = itertools.takewhile(lambda x:x[0] < 128, enumerate(dict_method()))
+        return dict([[repr(k), truncate_str(repr(input[k]))] for i, k in dict_keys])
+    elif isinstance(input, (tuple, list, set, frozenset)):
+        return [truncate_str(repr(v)) for i, v in
+                itertools.takewhile(lambda x:x[0] < 128, enumerate(input))]
+    elif isinstance(input, basestring):
+        return truncate_str(input)
+    else:
+        to_return = repr(input)
+        return truncate_str(to_return)
+
+def shorten_filename(frame):
+    filename = frame.filename
+    if frame.module:
+        try:
+            # shorten filenames
+            package = frame.module.split('.', 1)[0]
+            base_filename = sys.modules[package].__file__
+            to_trunc = base_filename.rsplit(os.sep, 2)[0]
+            filename = filename.split(to_trunc, 1)[-1][1:]
+        except Exception as e:
+            pass
+    return filename
 
 class cached_property(object):
     """A decorator that converts a function into a lazy property.  The
@@ -240,14 +278,57 @@ class Traceback(object):
     def plaintext(self):
         result = ['Traceback (most recent call last):']
         for frame in self.frames:
-            result.append('File "%s", line %s, in %s' % 
-                    (frame.filename, frame.lineno, frame.function_name,))
+            result.append('File "%s", line %s, in %s' %
+                    (shorten_filename(frame), frame.lineno, frame.function_name,))
             result.append('    %s' % frame.current_line.strip())
         result.append('%s' % self.exception)
         return '\n'.join(result)
 
     id = property(lambda x: id(x))
 
+    def frameinfo(self, include_vars=False):
+        """Dict representing frame and variables"""
+        result = []
+        id_list = []
+        for frame in self.frames:
+            if frame.hide:
+                continue
+            entry = {'file':shorten_filename(frame),
+                     'line':frame.lineno,
+                     'fn':frame.function_name,
+                     'cline':frame.current_line.strip(),
+                     'vars':[]}
+
+            # in some situations frame.locals might be not something we expect
+            if include_vars:
+                if not isinstance(frame.locals, dict):
+                    entry['vars'].append({'unknown': '<uninspectable>'})
+                    continue
+                for k, v in frame.locals.iteritems():
+                    if id(v) not in id_list:
+                        id_list.append(id(v))
+                    else:
+                        continue
+                    try:
+                        if k == 'self':
+                            entry['vars'].append(['self.' + v.__class__.__name__,
+                                                  serialize_to_unicode(v)])
+                        else:
+                            if k == 'environ':
+                                entry['vars'].append([k, '<environ-skipped>'])
+                                continue
+                            entry['vars'].append([k, serialize_to_unicode(v)])
+                    except Exception as e:
+                        entry['vars'].append([k, '<unserializable>'])
+            result.append(entry)
+        result.append({'file':'',
+                     'line':'',
+                     'fn':'',
+                     'cline':'%s' % self.exception,
+                     'vars':{}}
+                      )
+        return result
+
 
 class Frame(object):
     """A single frame in a traceback."""
@@ -262,8 +343,9 @@ class Frame(object):
         if fn[-4:] in ('.pyo', '.pyc'):
             fn = fn[:-1]
         # if it's a file on the file system resolve the real filename.
-        if os.path.isfile(fn):
-            fn = os.path.realpath(fn)
+# we actually dont want that or it will break filename shortening
+#        if os.path.isfile(fn):
+#            fn = os.path.realpath(fn)
         self.filename = fn
         self.module = self.globals.get('__name__')
         self.loader = self.globals.get('__loader__')
